@@ -23,6 +23,10 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#if !defined(__CLUTTER_GST_H_INSIDE__) && !defined(CLUTTER_GST_COMPILATION)
+#error "Only <clutter-gst/clutter-gst.h> can be included directly."
+#endif
+
 /**
  * SECTION:clutter-gst-version
  * @short_description: Versioning Macros
@@ -38,14 +42,14 @@
  *
  * ClutterGst major version (e.g. "1", if %CLUTTER_GST_VERSION is "1.2.3")
  */
-#define CLUTTER_GST_MAJOR_VERSION   (0)
+#define CLUTTER_GST_MAJOR_VERSION   (1)
 
 /**
  * CLUTTER_GST_MINOR_VERSION:
  *
  * ClutterGst minor version (e.g. "2", if %CLUTTER_GST_VERSION is "1.2.3")
  */
-#define CLUTTER_GST_MINOR_VERSION   (10)
+#define CLUTTER_GST_MINOR_VERSION   (0)
 
 /**
  * CLUTTER_GST_MICRO_VERSION:
@@ -59,14 +63,14 @@
  *
  * ClutterGst full version (e.g. "1.2.3")
  */
-#define CLUTTER_GST_VERSION         (0.10.0)
+#define CLUTTER_GST_VERSION         (1.0.0)
 
 /**
  * CLUTTER_GST_VERSION_S:
  *
  * ClutterGst full version, encoded as a string.
  */
-#define CLUTTER_GST_VERSION_S       "0.10.0"
+#define CLUTTER_GST_VERSION_S       "1.0.0"
 
 /**
  * CLUTTER_GST_VERSION_HEX:
