@@ -7,8 +7,10 @@
  *                               video stream.
  *
  * Authored By Matthew Allum  <mallum@openedhand.com>
+ *             Damien Lespiau <damien.lespiau@intel.com>
  *
  * Copyright (C) 2006 OpenedHand
+ * Copyright (C) 2010 Intel Corporation
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -37,15 +39,19 @@
 #include "config.h"
 #endif
 
-#include "clutter-gst-video-texture.h"
-#include "clutter-gst-video-sink.h"
+#include <string.h>
 
-#include <gst/gst.h>
 #include <glib.h>
+#include <gio/gio.h>
+#include <gst/gst.h>
+
+#include "clutter-gst-debug.h"
+#include "clutter-gst-video-sink.h"
+#include "clutter-gst-video-texture.h"
 
 struct _ClutterGstVideoTexturePrivate
 {
-  GstElement *playbin;
+  GstElement *pipeline;
 
   gchar *uri;
 
@@ -93,6 +99,111 @@ tick_timeout (gpointer data)
   return TRUE;
 }
 
+static void
+autoload_subtitle (ClutterGstVideoTexture *video_texture,
+                   const gchar            *uri)
+{
+  ClutterGstVideoTexturePrivate *priv = video_texture->priv;
+  gchar *path, *dot, *subtitle_path;
+  GFile *video;
+  guint i;
+
+  static const char subtitles_extensions[][4] =
+    {
+      "sub", "SUB",
+      "srt", "SRT",
+      "smi", "SMI",
+      "ssa", "SSA",
+      "ass", "ASS",
+      "asc", "ASC"
+    };
+
+  /* do not try to look for subtitle files if the video file is not mounted
+   * locally */
+  if (!g_str_has_prefix (uri, "file://"))
+    return;
+
+  /* Retrieve the absolute path of the video file */
+  video = g_file_new_for_uri (uri);
+  path = g_file_get_path (video);
+  g_object_unref (video);
+  if (path == NULL)
+    return;
+
+  /* Put a '\0' after the dot of the extension */
+  dot = strrchr (path, '.');
+  if (dot == NULL) {
+    g_free (path);
+    return;
+  }
+  *++dot = '\0';
+
+  /* we can't use path as the temporary buffer for the paths of the potential
+   * subtitle files as we may not have enough room there */
+  subtitle_path = g_malloc (strlen (path) + 1 + 4);
+  strcpy (subtitle_path, path);
+
+  /* reuse dot to point to the first byte of the extension of subtitle_path */
+  dot = subtitle_path + (dot - path);
+
+  for (i = 0; i < G_N_ELEMENTS (subtitles_extensions); i++)
+    {
+      GFile *candidate;
+
+      memcpy (dot, subtitles_extensions[i], 4);
+      candidate = g_file_new_for_path (subtitle_path);
+      if (g_file_query_exists (candidate, NULL))
+        {
+          gchar *suburi;
+
+          suburi = g_file_get_uri (candidate);
+
+          CLUTTER_GST_NOTE (MEDIA, "found subtitle: %s", suburi);
+
+          g_object_set (priv->pipeline, "suburi", suburi, NULL);
+          g_free (suburi);
+
+          g_object_unref (candidate);
+          break;
+        }
+
+      g_object_unref (candidate);
+    }
+
+  g_free (path);
+  g_free (subtitle_path);
+}
+
+static void
+query_duration (ClutterGstVideoTexture *video_texture)
+{
+  ClutterGstVideoTexturePrivate *priv = video_texture->priv;
+  gboolean success;
+  GstFormat format = GST_FORMAT_TIME;
+  gint64 duration;
+  gdouble new_duration, difference;
+
+  success = gst_element_query_duration (priv->pipeline, &format, &duration);
+  if (G_UNLIKELY (success != TRUE))
+    return;
+
+  new_duration = (gdouble) duration / GST_SECOND;
+
+  /* while we store the new duration if it sligthly changes, the duration
+   * signal is sent only if the new duration is at least one second different
+   * from the old one (as the duration signal is mainly used to update the
+   * time displayed in a UI */
+  difference = ABS (priv->duration - new_duration);
+  if (difference > 1e-3)
+    {
+      CLUTTER_GST_NOTE (MEDIA, "duration: %.02f", new_duration);
+      priv->duration = new_duration;
+
+      if (difference > 1.0)
+        g_object_notify (G_OBJECT (video_texture), "duration");
+    }
+}
+
 static void
 set_uri (ClutterGstVideoTexture *video_texture,
          const gchar            *uri)
@@ -101,7 +212,7 @@ set_uri (ClutterGstVideoTexture *video_texture,
   GObject *self = G_OBJECT (video_texture);
   GstState state, pending;
 
-  if (!priv->playbin)
+  if (!priv->pipeline)
     return;
 
   g_free (priv->uri);
@@ -120,6 +231,9 @@ set_uri (ClutterGstVideoTexture *video_texture,
           priv->tick_timeout_id =
             g_timeout_add (TICK_TIMEOUT * 1000, tick_timeout, self);
         }
+
+      /* try to load subtitles based on the uri of the file */
+      autoload_subtitle (video_texture, uri);
     }
   else 
     {
@@ -135,20 +249,22 @@ set_uri (ClutterGstVideoTexture *video_texture,
   priv->can_seek = FALSE;
   priv->duration = 0.0;
 
-  gst_element_get_state (priv->playbin, &state, &pending, 0);
+  gst_element_get_state (priv->pipeline, &state, &pending, 0);
 
   if (pending)
     state = pending;
 
-  gst_element_set_state (priv->playbin, GST_STATE_NULL);
+  gst_element_set_state (priv->pipeline, GST_STATE_NULL);
 
-  g_object_set (priv->playbin, "uri", uri, NULL);
+  CLUTTER_GST_NOTE (MEDIA, "setting URI: %s", uri);
+
+  g_object_set (priv->pipeline, "uri", uri, NULL);
 
   /*
    * Restore state.
    */
   if (uri)
-    gst_element_set_state (priv->playbin, state);
+    gst_element_set_state (priv->pipeline, state);
 
   /*
    * Emit notififications for all these to make sure UI is not showing
@@ -166,9 +282,11 @@ set_playing (ClutterGstVideoTexture *video_texture,
 {
   ClutterGstVideoTexturePrivate *priv = video_texture->priv;
 
-  if (!priv->playbin)
+  if (!priv->pipeline)
     return;
 
+  CLUTTER_GST_NOTE (MEDIA, "set playing: %d", playing);
+
   if (priv->uri) 
     {
       GstState state = GST_STATE_PAUSED;
@@ -176,7 +294,7 @@ set_playing (ClutterGstVideoTexture *video_texture,
       if (playing)
 	state = GST_STATE_PLAYING;
 
-      gst_element_set_state (priv->playbin, state);
+      gst_element_set_state (priv->pipeline, state);
     } 
   else 
     {
@@ -193,16 +311,21 @@ get_playing (ClutterGstVideoTexture *video_texture)
 {
   ClutterGstVideoTexturePrivate *priv = video_texture->priv;
   GstState state, pending;
+  gboolean playing;
 
-  if (!priv->playbin)
+  if (!priv->pipeline)
     return FALSE;
   
-  gst_element_get_state (priv->playbin, &state, &pending, 0);
+  gst_element_get_state (priv->pipeline, &state, &pending, 0);
   
   if (pending)
-    return (pending == GST_STATE_PLAYING);
+    playing = (pending == GST_STATE_PLAYING);
   else
-    return (state == GST_STATE_PLAYING);
+    playing = (state == GST_STATE_PLAYING);
+
+  CLUTTER_GST_NOTE (MEDIA, "get playing: %d", playing);
+
+  return playing;
 }
 
 static void
@@ -214,19 +337,21 @@ set_progress (ClutterGstVideoTexture *video_texture,
   GstQuery *duration_q;
   gint64 position;
 
-  if (!priv->playbin)
+  if (!priv->pipeline)
     return;
 
-  gst_element_get_state (priv->playbin, &state, &pending, 0);
+  CLUTTER_GST_NOTE (MEDIA, "set progress: %.02f", progress);
+
+  gst_element_get_state (priv->pipeline, &state, &pending, 0);
 
   if (pending)
     state = pending;
 
-  gst_element_set_state (priv->playbin, GST_STATE_PAUSED);
+  gst_element_set_state (priv->pipeline, GST_STATE_PAUSED);
 
   duration_q = gst_query_new_duration (GST_FORMAT_TIME);
 
-  if (gst_element_query (priv->playbin, duration_q))
+  if (gst_element_query (priv->pipeline, duration_q))
     {
       gint64 duration = 0;
 
@@ -239,7 +364,7 @@ set_progress (ClutterGstVideoTexture *video_texture,
 
   gst_query_unref (duration_q);
 
-  gst_element_seek (priv->playbin,
+  gst_element_seek (priv->pipeline,
 		    1.0,
 		    GST_FORMAT_TIME,
 		    GST_SEEK_FLAG_FLUSH,
@@ -247,7 +372,7 @@ set_progress (ClutterGstVideoTexture *video_texture,
 		    position,
 		    0, 0);
 
-  gst_element_set_state (priv->playbin, state);
+  gst_element_set_state (priv->pipeline, state);
 
   g_object_notify (G_OBJECT (video_texture), "progress");
 }
@@ -259,14 +384,14 @@ get_progress (ClutterGstVideoTexture *video_texture)
   GstQuery *position_q, *duration_q;
   gdouble progress;
 
-  if (!priv->playbin)
+  if (!priv->pipeline)
     return 0.0;
 
   position_q = gst_query_new_position (GST_FORMAT_TIME);
   duration_q = gst_query_new_duration (GST_FORMAT_TIME);
 
-  if (gst_element_query (priv->playbin, position_q) &&
-      gst_element_query (priv->playbin, duration_q))
+  if (gst_element_query (priv->pipeline, position_q) &&
+      gst_element_query (priv->pipeline, duration_q))
     {
       gint64 position, duration;
 
@@ -283,6 +408,8 @@ get_progress (ClutterGstVideoTexture *video_texture)
   gst_query_unref (position_q);
   gst_query_unref (duration_q);
 
+  CLUTTER_GST_NOTE (MEDIA, "get progress: %.02f", progress);
+
   return progress;
 }
 
@@ -292,11 +419,13 @@ set_audio_volume (ClutterGstVideoTexture *video_texture,
 {
   ClutterGstVideoTexturePrivate *priv = video_texture->priv;
 
-  if (!priv->playbin)
+  if (!priv->pipeline)
     return;
 
+  CLUTTER_GST_NOTE (MEDIA, "set volume: %.02f", volume);
+
   /* the :volume property is in the [0, 10] interval */
-  g_object_set (G_OBJECT (priv->playbin), "volume", volume * 10.0, NULL);
+  g_object_set (G_OBJECT (priv->pipeline), "volume", volume * 10.0, NULL);
   g_object_notify (G_OBJECT (video_texture), "audio-volume");
 }
 
@@ -306,13 +435,17 @@ get_audio_volume (ClutterGstVideoTexture *video_texture)
   ClutterGstVideoTexturePrivate *priv = video_texture->priv;
   gdouble volume = 0.0;
 
-  if (!priv->playbin)
+  if (!priv->pipeline)
     return 0.0;
 
   /* the :volume property is in the [0, 10] interval */
-  g_object_get (priv->playbin, "volume", &volume, NULL);
+  g_object_get (priv->pipeline, "volume", &volume, NULL);
 
-  return CLAMP (volume / 10.0, 0.0, 1.0);
+  volume = CLAMP (volume / 10.0, 0.0, 1.0);
+
+  CLUTTER_GST_NOTE (MEDIA, "get volume: %.02f", volume);
+
+  return volume;
 }
 
 static void
@@ -332,11 +465,11 @@ clutter_gst_video_texture_dispose (GObject *object)
   /* FIXME: flush an errors off bus ? */
   /* gst_bus_set_flushing (priv->bus, TRUE); */
 
-  if (priv->playbin) 
+  if (priv->pipeline) 
     {
-      gst_element_set_state (priv->playbin, GST_STATE_NULL);
-      gst_object_unref (GST_OBJECT (priv->playbin));
-      priv->playbin = NULL;
+      gst_element_set_state (priv->pipeline, GST_STATE_NULL);
+      gst_object_unref (GST_OBJECT (priv->pipeline));
+      priv->pipeline = NULL;
     }
 
   if (priv->tick_timeout_id > 0) 
@@ -491,6 +624,8 @@ bus_message_eos_cb (GstBus                 *bus,
 {
   g_object_notify (G_OBJECT (video_texture), "progress");
 
+  CLUTTER_GST_NOTE (MEDIA, "EOS");
+
   g_signal_emit_by_name (video_texture, "eos");
 }
 
@@ -515,6 +650,8 @@ bus_message_buffering_cb (GstBus                 *bus,
                                  0.0,
                                  1.0);
 
+      CLUTTER_GST_NOTE (MEDIA, "buffer-fill: %.02f", priv->buffer_fill);
+
       g_object_notify (G_OBJECT (video_texture), "buffer-fill");
     }
 }
@@ -524,17 +661,15 @@ bus_message_duration_cb (GstBus                 *bus,
                          GstMessage             *message,
                          ClutterGstVideoTexture *video_texture)
 {
-  ClutterGstVideoTexturePrivate *priv = video_texture->priv;
-  GstFormat format;
   gint64 duration;
 
-  gst_message_parse_duration (message, &format, &duration);
-  if (format != GST_FORMAT_TIME)
+  /* GstElements send a duration message on the bus with GST_CLOCK_TIME_NONE
+   * as duration to signal a new duration */
+  gst_message_parse_duration (message, NULL, &duration);
+  if (G_UNLIKELY (duration != GST_CLOCK_TIME_NONE))
     return;
-  
-  priv->duration = (gdouble) duration / GST_SECOND;
 
-  g_object_notify (G_OBJECT (video_texture), "duration");
+  query_duration (video_texture);
 }
 
 static void
@@ -547,7 +682,7 @@ bus_message_state_change_cb (GstBus                 *bus,
   gpointer src;
 
   src = GST_MESSAGE_SRC (message);
-  if (src != priv->playbin)
+  if (src != priv->pipeline)
     return;
 
   gst_message_parse_state_changed (message, &old_state, &new_state, NULL);
@@ -560,7 +695,7 @@ bus_message_state_change_cb (GstBus                 *bus,
       /* Determine whether we can seek */
       query = gst_query_new_seeking (GST_FORMAT_TIME);
 
-      if (gst_element_query (priv->playbin, query))
+      if (gst_element_query (priv->pipeline, query))
         {
           gboolean can_seek = FALSE;
 
@@ -573,7 +708,7 @@ bus_message_state_change_cb (GstBus                 *bus,
       else
         {
 	  /* could not query for ability to seek by querying the
-           * playbin; let's crudely try by using the URI
+           * pipeline; let's crudely try by using the URI
 	   */
 	  if (priv->uri && g_str_has_prefix (priv->uri, "http://"))
             priv->can_seek = FALSE;
@@ -583,22 +718,11 @@ bus_message_state_change_cb (GstBus                 *bus,
 
       gst_query_unref (query);
 
+      CLUTTER_GST_NOTE (MEDIA, "can-seek: %d", priv->can_seek);
+
       g_object_notify (G_OBJECT (video_texture), "can-seek");
-      
-      /* Determine the duration */
-      query = gst_query_new_duration (GST_FORMAT_TIME);
 
-      if (gst_element_query (priv->playbin, query)) 
-	{
-	  gint64 duration;
-
-	  gst_query_parse_duration (query, NULL, &duration);
-	  priv->duration = (gdouble) duration / GST_SECOND;
-
-	  g_object_notify (G_OBJECT (video_texture), "duration");
-	}
-
-      gst_query_unref (query);
+      query_duration (video_texture);
     }
 }
 
@@ -609,10 +733,10 @@ lay_pipeline (ClutterGstVideoTexture *video_texture)
   GstElement *audio_sink = NULL;
   GstElement *video_sink = NULL;
 
-  priv->playbin = gst_element_factory_make ("playbin", "playbin");
-  if (!priv->playbin) 
+  priv->pipeline = gst_element_factory_make ("playbin2", "pipeline");
+  if (!priv->pipeline) 
     {
-      g_critical ("Unable to create playbin element");
+      g_critical ("Unable to create playbin2 element");
       return FALSE;
     }
 
@@ -638,10 +762,11 @@ lay_pipeline (ClutterGstVideoTexture *video_texture)
 
   video_sink = clutter_gst_video_sink_new (CLUTTER_TEXTURE (video_texture));
   g_object_set (G_OBJECT (video_sink), "qos", TRUE, "sync", TRUE, NULL);
-  g_object_set (G_OBJECT (priv->playbin),
-		"video-sink", video_sink,
-		"audio-sink", audio_sink,
-		NULL);
+  g_object_set (G_OBJECT (priv->pipeline),
+                "video-sink", video_sink,
+                "audio-sink", audio_sink,
+                "subtitle-font-desc", "Sans 16",
+                NULL);
 
   return TRUE;
 }
@@ -663,7 +788,7 @@ clutter_gst_video_texture_init (ClutterGstVideoTexture *video_texture)
       return;
     }
 
-  bus = gst_pipeline_get_bus (GST_PIPELINE (priv->playbin));
+  bus = gst_pipeline_get_bus (GST_PIPELINE (priv->pipeline));
 
   gst_bus_add_signal_watch (bus);
 
@@ -691,6 +816,11 @@ clutter_gst_video_texture_init (ClutterGstVideoTexture *video_texture)
  *
  * Creates a video texture.
  *
+ * <note>This function has to be called from Clutter's main thread. While
+ * GStreamer will spawn threads to do its work, we want all the GL calls to
+ * happen in the same thread. Clutter-gst knows which thread it is by
+ * assuming this constructor is called from the Clutter thread.</note>
+ *
  * Return value: the newly created video texture actor
  */
 ClutterActor*
@@ -702,18 +832,18 @@ clutter_gst_video_texture_new (void)
 }
 
 /**
- * clutter_gst_video_texture_get_playbin:
+ * clutter_gst_video_texture_get_pipeline:
  * @texture: a #ClutterGstVideoTexture
  *
- * Retrieves the #GstElement used by the @texture, for direct use with
+ * Retrieves the #GstPipeline used by the @texture, for direct use with
  * GStreamer API.
  *
- * Return value: the playbin element used by the video texture
+ * Return value: the pipeline element used by the video texture
  */
 GstElement *
-clutter_gst_video_texture_get_playbin (ClutterGstVideoTexture *texture)
+clutter_gst_video_texture_get_pipeline (ClutterGstVideoTexture *texture)
 {
   g_return_val_if_fail (CLUTTER_GST_IS_VIDEO_TEXTURE (texture), NULL);
 
-  return texture->priv->playbin;
+  return texture->priv->pipeline;
 }
